package com.chinaitop.depot.device.controller;

import com.chinaitop.depot.device.model.*;
import com.chinaitop.depot.device.service.DFacilitiesBasicsService;
import com.chinaitop.depot.device.service.DFacilitiesMaintainService;
import com.chinaitop.depot.device.service.DFacilitiesRepairService;
import com.chinaitop.depot.device.service.DSblxxjyService;
import com.chinaitop.depot.utils.GeneratingEncodUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author yu.ren
 * @description: 1:设备,2:设施,3:器材
 * @create 2022-04-16 11:26
 */
@RestController
@RequestMapping("/facilities")
@Api(value = "DFacilitiesController", description = "设施管理")
public class DFacilitiesController {
    @Resource
    private DFacilitiesBasicsService dFacilitiesBasicsService;
    @Resource
    private DFacilitiesMaintainService dFacilitiesMaintainService;
    @Resource
    private DFacilitiesRepairService dFacilitiesRepairService;

    /**
     * 分页获取设施基础信息
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @return
     */
    @RequestMapping(value = "/basics/pageList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询设施基础信息", notes = "查询设施基础信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query")
    })
    public PageInfo<DFacilitiesBasics> getPageInfoList(Integer pageNum, Integer pageSize, Integer orgId) {
        DFacilitiesBasicsExample example = new DFacilitiesBasicsExample();
        DFacilitiesBasicsExample.Criteria criteria = example.createCriteria();
        // 当前登录人组织代码
        if (orgId != null) {
            criteria.andOrgIdEqualTo(orgId);
        }

        //排序
        example.setOrderByClause("create_time desc");
        // 分页
        if (pageNum != null && pageSize != null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<DFacilitiesBasics> list = dFacilitiesBasicsService.getList(example);
        PageInfo<DFacilitiesBasics> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    @RequestMapping(value = "/basics/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存/编辑设施基础信息", notes = "设施基础信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dFacilitiesBasicsJson", value = "设施基础信息数据", paramType = "form")
    })
    public Map<String, Object> saveBasics(String dFacilitiesBasicsJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
            DFacilitiesBasics dFacilitiesBasics = mapper.readValue(dFacilitiesBasicsJson, DFacilitiesBasics.class);
            if (dFacilitiesBasics.getId()==null) {
                dFacilitiesBasics.setCreateTime(new Date()); // 创建时间
                dFacilitiesBasicsService.save(dFacilitiesBasics); // 新增
            } else {
                dFacilitiesBasicsService.update(dFacilitiesBasics); // 修改
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    @RequestMapping(value = "/basics/remove", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "根据id删除设施基础信息", notes = "根据id删除设施基础信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public Map<String, Object> remove(Integer id) {
        Map<String, Object> modelMap = new HashMap<>();
        try {
            if (id != null) {
                dFacilitiesBasicsService.remove(id);
                modelMap.put("status", "success");
            } else {
                modelMap.put("status", "error");
            }
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    /**
     * 获取设施基础信息列表
     *
     * @param orgId
     * @return
     */
    @RequestMapping(value = "/basics/list", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询设施基础信息", notes = "查询设施基础信息列表，下拉框设施列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query")
    })
    public List<DFacilitiesBasics> getBasicsList(Integer orgId) {
        DFacilitiesBasicsExample example = new DFacilitiesBasicsExample();
        DFacilitiesBasicsExample.Criteria criteria = example.createCriteria();
        // 当前登录人组织代码
        if (orgId != null) {
            criteria.andOrgIdEqualTo(orgId);
        }
        //排序
        example.setOrderByClause("create_time desc");
        List<DFacilitiesBasics> list = dFacilitiesBasicsService.getList(example);
        return list;
    }

    @RequestMapping(value = "/basics/scrap", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "设施报废", notes = "设施报废")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dFacilitiesBasicsJson", value = "设施基础信息数据", paramType = "form")
    })
    public Map<String, Object> scrap(String dFacilitiesBasicsJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
            DFacilitiesBasics dFacilitiesBasics = mapper.readValue(dFacilitiesBasicsJson, DFacilitiesBasics.class);
            if (dFacilitiesBasics.getId()!=null) {
                dFacilitiesBasicsService.scrap(dFacilitiesBasics);
            } else {
                modelMap.put("status", "error no id");
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    /**
     * 分页获取设施保养信息
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @return
     */
    @RequestMapping(value = "/maintain/pageList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询设施保养信息", notes = "查询设施保养信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query")
    })
    public PageInfo<DFacilitiesMaintain> getMaintainPageInfoList(Integer pageNum, Integer pageSize, Integer orgId) {
        DFacilitiesMaintainExample example = new DFacilitiesMaintainExample();
        DFacilitiesMaintainExample.Criteria criteria = example.createCriteria();
        // 当前登录人组织代码
        if (orgId != null) {
            criteria.andOrgIdEqualTo(orgId);
        }

        //排序
        example.setOrderByClause("create_time desc");
        // 分页
        if (pageNum != null && pageSize != null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<DFacilitiesMaintain> list = dFacilitiesMaintainService.getList(example);
        PageInfo<DFacilitiesMaintain> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    @RequestMapping(value = "/maintain/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存/编辑设施保养信息", notes = "设施保养信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dFacilitiesMaintainJson", value = "设施保养信息", paramType = "form")
    })
    public Map<String, Object> saveMaintain(String dFacilitiesMaintainJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
            DFacilitiesMaintain dFacilitiesMaintain = mapper.readValue(dFacilitiesMaintainJson, DFacilitiesMaintain.class);
            if (dFacilitiesMaintain.getId()==null) {
                dFacilitiesMaintain.setCreateTime(new Date()); // 创建时间
                dFacilitiesMaintainService.save(dFacilitiesMaintain); // 新增
            } else {
                dFacilitiesMaintainService.update(dFacilitiesMaintain); // 修改
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    @RequestMapping(value = "/maintain/remove", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "根据id删除设施保养信息", notes = "根据id删除设施保养信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public Map<String, Object> removeMaintain(Integer id) {
        Map<String, Object> modelMap = new HashMap<>();
        try {
            if (id != null) {
                dFacilitiesMaintainService.remove(id);
                modelMap.put("status", "success");
            } else {
                modelMap.put("status", "error");
            }
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    /**
     * 分页获取设施维修信息
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @return
     */
    @RequestMapping(value = "/repair/pageList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询设施维修信息", notes = "查询设施维修信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query")
    })
    public PageInfo<DFacilitiesRepair> getRepairPageInfoList(Integer pageNum, Integer pageSize, Integer orgId) {
        DFacilitiesRepairExample example = new DFacilitiesRepairExample();
        DFacilitiesRepairExample.Criteria criteria = example.createCriteria();
        // 当前登录人组织代码
        if (orgId != null) {
            criteria.andOrgIdEqualTo(orgId);
        }

        //排序
        example.setOrderByClause("create_time desc");
        // 分页
        if (pageNum != null && pageSize != null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<DFacilitiesRepair> list = dFacilitiesRepairService.getList(example);
        PageInfo<DFacilitiesRepair> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    @RequestMapping(value = "/repair/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存/编辑设施维修信息", notes = "设施维修信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dFacilitiesRepairJson", value = "设施维修信息", paramType = "form")
    })
    public Map<String, Object> saveRepair(String dFacilitiesRepairJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
            DFacilitiesRepair dFacilitiesRepair = mapper.readValue(dFacilitiesRepairJson, DFacilitiesRepair.class);
            if (dFacilitiesRepair.getId()==null) {
                dFacilitiesRepair.setCreateTime(new Date()); // 创建时间
                dFacilitiesRepairService.save(dFacilitiesRepair); // 新增
            } else {
                dFacilitiesRepairService.update(dFacilitiesRepair); // 修改
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    @RequestMapping(value = "/repair/remove", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "根据id删除设施维修信息", notes = "根据id删除设施维修信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
    })
    public Map<String, Object> removeRepair(Integer id) {
        Map<String, Object> modelMap = new HashMap<>();
        try {
            if (id != null) {
                dFacilitiesRepairService.remove(id);
                modelMap.put("status", "success");
            } else {
                modelMap.put("status", "error");
            }
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

}
