package com.chinaitop.depot.device.controller;

import com.chinaitop.depot.device.model.DSbxxsjy;
import com.chinaitop.depot.device.model.DSbxxsjyExample;
import com.chinaitop.depot.device.service.DSblxxjyService;
import com.chinaitop.depot.utils.GeneratingEncodUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author qingsong.han
 * @description: 1:设备,2:设施,3:器材
 * @create 2020-03-20 12:44
 */
@RestController
@RequestMapping("/devicePutStorage")
@Api(value = "DSblxxjyController", description = "设备器材设施入库")
public class DSblxxjyController {
    @Resource
    private DSblxxjyService sblxxjyService;

    /**
     * 分页获取器材设备设施信息
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @param ssfl
     * @param sbyqmc
     * @return
     */
    @RequestMapping(value = "/getPageInfoList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询入库设备信息", notes = "查询入库设备列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query")
    })
    public PageInfo<DSbxxsjy> getPageInfoList(Integer pageNum, Integer pageSize, Integer orgId, String ssfl, String sbyqmc, String kddm) {
        DSbxxsjyExample example = new DSbxxsjyExample();
        DSbxxsjyExample.Criteria criteria = example.createCriteria();
        // szdwdm 所在单位代码
        if (orgId != null) {
            criteria.andSzdwdmEqualTo(orgId.toString());
        }
        // ssfl 1:设备,2:设施,3:器材
        if (StringUtils.isNotBlank(ssfl)) {
            if ("5475".equals(ssfl)) // 器材类型
                ssfl = "3";
            if ("5476".equals(ssfl)) // 设备类型
                ssfl = "1";
            criteria.andSsflEqualTo(ssfl);
        }
        // sbyqmc 设备仪器名称
        if (StringUtils.isNotBlank(sbyqmc)) {
            criteria.andSbyqmcEqualTo(sbyqmc);
        }
        // 库点编号
        /*if (StringUtils.isNotBlank(kddm)) {
            example.or().andKddmEqualTo(kddm);
        }*/
        //排序
        example.setOrderByClause("create_date desc");
        // 分页
        if (pageNum != null && pageSize != null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<DSbxxsjy> list = sblxxjyService.getList(example);
        PageInfo<DSbxxsjy> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存数据", notes = "保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dSbxxsjyJson", value = "主表数据", paramType = "form")
    })
    public Map<String, Object> save(String dSbxxsjyJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
            DSbxxsjy dSbxxsjy = mapper.readValue(dSbxxsjyJson, DSbxxsjy.class);
            dSbxxsjy.setWhsj(dSbxxsjy.getCreateDate()); // 维护时间
            // 生成设备器材编码
            if (StringUtils.isBlank(dSbxxsjy.getSbbh())) {
                dSbxxsjy.setSbbh(GeneratingEncodUtils.gotSbbh(dSbxxsjy.getSzdwdm()));
                sblxxjyService.save(dSbxxsjy); // 新增
            } else {
                sblxxjyService.update(dSbxxsjy); // 修改
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    @RequestMapping(value = "/remove", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "根据id删除信息", notes = "根据id删除信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键id", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织机构id", paramType = "query")
    })
    public Map<String, Object> remove(String sbbh, Integer orgId) {
        Map<String, Object> modelMap = new HashMap<>();
        try {
            if (StringUtils.isNotBlank(sbbh) && orgId != null) {
                sblxxjyService.remove(sbbh, orgId.toString());
                modelMap.put("status", "success");
            } else {
                modelMap.put("status", "error");
            }
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    @RequestMapping(value = "/getSbCode", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询有效入库设备", notes = "查询有效入库设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query"),
            @ApiImplicitParam(name = "sbbh", value = "设备编码", paramType = "query")
    })
    public List<String> getSbCode(Integer orgId, String sbbh) {
        DSbxxsjyExample example = new DSbxxsjyExample();
        DSbxxsjyExample.Criteria criteria = example.createCriteria();
        // szdwdm 所在单位代码
        if (orgId != null) {
            criteria.andSzdwdmEqualTo(orgId.toString());
        }
        criteria.andSbslGreaterThan((long) 0);
        criteria.andStatusEqualTo("1");
        //排序
        example.setOrderByClause("create_date desc");
        List<DSbxxsjy> list = sblxxjyService.getList(example);
        List<String> collect = list.stream().map(DSbxxsjy::getSbbh).collect(Collectors.toList());
        return collect;
    }

    @RequestMapping(value = "/getBySbCode", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询有效入库设备", notes = "查询有效入库设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query"),
            @ApiImplicitParam(name = "sbbh", value = "设备编码", paramType = "query")
    })
    public DSbxxsjy getBySbCode(Integer orgId, String sbbh) {
        DSbxxsjyExample example = new DSbxxsjyExample();
        DSbxxsjyExample.Criteria criteria = example.createCriteria();
        // szdwdm 所在单位代码
        if (orgId != null) {
            criteria.andSzdwdmEqualTo(orgId.toString());
        }
        if (StringUtils.isNotBlank(sbbh)) {
            criteria.andSbbhEqualTo(sbbh);
        }
        List<DSbxxsjy> list = sblxxjyService.getList(example);

        return list.get(0);
    }

    @RequestMapping(value = "/outUpdate", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存数据", notes = "保存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sbbh", value = "主表数据", paramType = "form")
    })
    public Map<String, Object> outUpdate(String sbbh) {
        Map<String, Object> modelMap = new HashMap<>();
        try {
            //根据设备编号获取入库表的该设备信息
            DSbxxsjyExample example = new DSbxxsjyExample();
            DSbxxsjyExample.Criteria criteria = example.createCriteria();
            criteria.andSbbhEqualTo(sbbh);
            DSbxxsjy dSbxxsjy = sblxxjyService.getList(example).get(0);
            dSbxxsjy.setStatus("0");
            sblxxjyService.update(dSbxxsjy); // 修改
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    @RequestMapping(value = "/getSbName", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据设备名称查询有效设备编码", notes = "根据设备名称查询有效设备编码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织代码", paramType = "query"),
            @ApiImplicitParam(name = "sbbh", value = "设备编码", paramType = "query")
    })
    public List<String> getSbName(Integer orgId, String sbyqmc) {
        DSbxxsjyExample example = new DSbxxsjyExample();
        DSbxxsjyExample.Criteria criteria = example.createCriteria();
        // szdwdm 所在单位代码
        if (orgId != null) {
            criteria.andSzdwdmEqualTo(orgId.toString());
        }
        if (StringUtils.isNotBlank(sbyqmc)) {
            criteria.andSbyqmcEqualTo(sbyqmc);
        }
        criteria.andSbslGreaterThan((long) 0);
        criteria.andStatusEqualTo("1");
        //排序
        example.setOrderByClause("create_date desc");
        List<DSbxxsjy> list = sblxxjyService.getList(example);
        List<String> collect = list.stream().map(DSbxxsjy::getSbbh).collect(Collectors.toList());
        return collect;
    }
}
