package com.chinaitop.depot.device.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import com.chinaitop.depot.device.model.*;
import com.chinaitop.depot.device.service.*;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.device.model.StorageDeviceinputExample.Criteria;
import com.chinaitop.depot.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value = {"/deviceGet"})
@Api(value = "StorageDeviceGetController", description = "器材设备领用")
public class StorageDeviceGetController {

    @Resource
    private StorageDeviceGetService deviceGetService;

    @Resource
    private StorageDeviceInputService deviceInputService;

    @Resource
    private StorageDeviceBackService deviceBackService;

    @Resource
    private BasicEquipmentEquipmentPoolService equipmentPoolService;

    @Resource
    private DSblxxjyService sblxxjyService;


    /**
     * 分页获取器材信息
     */
    @RequestMapping(value = "/getDeviceList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "获取领用信息", notes = "领用信息查询，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgName", value = "当前登录人组织名称", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "当前登录人组织名称", paramType = "query")
    })
    public PageInfo<StorageDeviceget> listCheckPage(Integer pageNum, Integer pageSize,
                                                    String deviceName, Integer deviceType,Integer orgId) {

        StorageDevicegetExample storageDevicegetExample = new StorageDevicegetExample();
        StorageDevicegetExample.Criteria criteria = storageDevicegetExample.createCriteria();

        if (ParameterUtil.isnotnull(deviceName)) {
            criteria.andDeviceNameLike(deviceName);
        }
        if (ParameterUtil.isnotnull(deviceType)) {
            criteria.andDeviceTypeEqualTo(deviceType);
        }
        if (ParameterUtil.isnotnull(orgId)) {
            criteria.andOrgIdEqualTo(orgId);
        }

        //按照id进行降序
        storageDevicegetExample.setOrderByClause("id desc");

        PageHelper.startPage(pageNum, pageSize);
        List<StorageDeviceget> list = deviceGetService.queryByDevice(storageDevicegetExample);
        PageInfo<StorageDeviceget> pageInfo = new PageInfo<StorageDeviceget>(list);
        return pageInfo;
    }

    /**
     * 根据主键id删除数据
     */
    @RequestMapping(value = "/remove", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "删除数据", notes = "根据id删除数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "form")
    })
    public Map<String, Object> remove(Integer id) {
        Map<String, Object> retMap = new HashMap<String, Object>();
        deviceInputService.remove(id);
        retMap.put("msg", "success");
        return retMap;
    }

    /**
     * 新增和修改的数据进行保存
     */
    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "保存数据", notes = "保存增加或修改的数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceGetJson", value = "新增数据", paramType = "form")
    })
    public Map<String, Object> save(String deviceGetJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        StorageDeviceget storageDeviceget = null;
        try {
            //保存领用数据
//        	ContentBean cb=(ContentBean)mapper.readValue(contentBeanJson, ContentBean.class);
            storageDeviceget = (StorageDeviceget) mapper.readValue(deviceGetJson, StorageDeviceget.class);
        	/*storageDeviceget.setDeviceName(cb.getDeviceName());//器材名称
        	storageDeviceget.setDeviceType(cb.getDeviceTypeId());//类型
        	storageDeviceget.setModel(cb.getModel());//规格型号
        	storageDeviceget.setNumber(cb.getNumber());//设备编号
        	storageDeviceget.setUseCount(cb.getUseCount());//可用的数量*/
            storageDeviceget.setBackCount(0);//归还数量
//        	storageDeviceget.setHouseId(storageDeviceget.getHouseId());//领用到的仓房
            deviceGetService.save(storageDeviceget);

            //根据设备编号获取入库表的该设备信息
            DSbxxsjyExample example = new DSbxxsjyExample();
            DSbxxsjyExample.Criteria criteria = example.createCriteria();
            criteria.andSbbhEqualTo(storageDeviceget.getNumber());
            DSbxxsjy dSbxxsjy = sblxxjyService.getList(example).get(0);
            dSbxxsjy.setSbsl(dSbxxsjy.getSbsl() - storageDeviceget.getGetCount());
            dSbxxsjy.setMqzt("6");
            sblxxjyService.update(dSbxxsjy);
//			List<StorageDeviceinput> inputList=deviceInputService.getUseCount(modelMap);
//        	inputList.get(0).setStatus(4);//领用状态
//        	inputList.get(0).setUseCount(inputList.get(0).getUseCount()-storageDeviceget.getGetCount());//数量减少
//        	deviceInputService.updateByPrimaryKeySelective(inputList.get(0));
            //改变入库的数量
        	/*StorageDeviceinputExample storageDeviceinputExample2 = new StorageDeviceinputExample();
        	Criteria criteria2 = storageDeviceinputExample2.createCriteria();
        	criteria2.andNumberEqualTo(cb.getNumber());
        	List<StorageDeviceinput> list=deviceInputService.queryByNumber(storageDeviceinputExample2);
        	StorageDeviceinputExample storageDeviceinputExample=new StorageDeviceinputExample();
        	StorageDeviceinputExample.Criteria criteria=storageDeviceinputExample.createCriteria();
        	criteria.andIdEqualTo(Integer.valueOf(storageDeviceget.getInputId()));
        	List<StorageDeviceinput> inputlist = deviceInputService.queryByDevice(storageDeviceinputExample);
            if(inputlist!=null){
            	StorageDeviceinput storageDeviceinput=list.get(0);
            	//使其状态改变为领用
            	storageDeviceinput.setStatus(4);
            	storageDeviceinput.setUseCount(storageDeviceinput.getUseCount()-storageDeviceget.getGetCount());
            	deviceInputService.save(storageDeviceinput);
            }*/
            modelMap.put("status", "success");
        } catch (IOException e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }


    /**
     * 根据id加载对应数据
     */
    @RequestMapping(value = "/loadDataById", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询数据", notes = "根据id查询数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "query")
    })
    public StorageDeviceinput loadDataById(int id) {
        if (id != 0) {
            return deviceInputService.loadDataById(id);
        }
        return new StorageDeviceinput();
    }

    /**
     * 获取入库的仓房名
     */
    @RequestMapping(value = "/getHouseList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "获取入库仓房名称", notes = "获取入库仓房名称")
    public List<HouseNameBean> getHouseList() {
        List<HouseNameBean> storageDeviceinput = deviceInputService.getHouseList();
        return storageDeviceinput;
    }

    /**
     * 获取仓房对应的类型
     */
    @RequestMapping(value = "/getTypeList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "获取仓房对应的类型", notes = "获取仓房对应的类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "depotId", value = "depotId", paramType = "query"),
    })
    public List<HouseTypeBean> getTypeList(Integer depotId) {
        List<HouseTypeBean> TypeNameList = deviceInputService.getTypeList(depotId);
        return TypeNameList;
    }

    /**
     * 获取仓房对应的类型的货架和名称getNameList
     */
    @RequestMapping(value = "/getNameList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "获取仓房对应的类型的货架和名称", notes = "获取仓房对应的类型的货架和名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "depotTypeId", value = "depotTypeId", paramType = "query"),
    })
    public List<ShelfContentBean> getNameList(Integer depotTypeId) {
        if (depotTypeId == 3) {
            depotTypeId = 5475;
        }
        if (depotTypeId == 1) {
            depotTypeId = 5476;
        }
        List<ShelfContentBean> getNameList = deviceInputService.getNameList(depotTypeId);
        return getNameList;
    }


    /**
     * 根据名称获取货架getShelfGetList
     */
    @RequestMapping(value = "/getShelfGetList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据名称获取货架", notes = "根据名称获取货架")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceName", value = "设备名称", paramType = "query"),
    })
    public List<ContentBean> getShelfGetList(String deviceName) {
        List<ContentBean> shelfGetList = deviceInputService.getShelfGetList(deviceName);
        return shelfGetList;
    }

    /**
     * 根据货架和名称获取规格型号getModelGetList
     */
    @RequestMapping(value = "/getModelGetList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据货架和名称获取规格型号", notes = "根据货架和名称获取规格型号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "shelfId", value = "货架", paramType = "query"),
            @ApiImplicitParam(name = "deviceName", value = "设备名称", paramType = "query")
    })
    public List<ContentBean> getModelGetList(String shelfId, String deviceName) {
        Map<String, Object> map = new HashMap<>();
        map.put("deviceName", deviceName);
        map.put("storageRack", shelfId);
        List<ContentBean> getModelList = deviceInputService.getModelGetList(map);
        return getModelList;
    }

    /**
     * 根据规格型号获取数量getNumberGetList
     */
    @RequestMapping(value = "/getNumberGetList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据规格型号获取数量", notes = "根据规格型号获取数量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "model", value = "规格型号", paramType = "query")
    })
    public List<StorageDeviceinput> getNumberGetList(String model) {
        List<StorageDeviceinput> getNumberList = deviceInputService.getNumberGetList(model);
        return getNumberList;
    }

    /**
     * 根据编号和规格获取可用数量getUseCountGetList
     */
    @RequestMapping(value = "/getUseCountGetList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据编号和规格获取可用数量", notes = "根据编号和规格获取可用数量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "model", value = "规格型号", paramType = "query"),
            @ApiImplicitParam(name = "number", value = "编号", paramType = "query")
    })
    public List<StorageDeviceinput> getUseCountGetList(String number, String model) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("number", number);
        map.put("model", model);
        List<StorageDeviceinput> getUseCountList = deviceInputService.getUseCount(map);
        return getUseCountList;
    }


    /**
     * 根据ID获取借取信息
     *
     * @throws Exception
     */
    @RequestMapping(value = "/getDeviceGetById", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据ID获取借取信息", notes = "根据ID获取借取信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "query")
    })
    public List<StorageDeviceget> getDeviceGetById(String id) throws Exception {
        StorageDeviceget storageDeviceget = null;
        if (id != null && !"".equals(id)) {
            storageDeviceget = deviceGetService.selectByPrimaryKey(Integer.valueOf(id));
        }
    	/*if(storageDeviceget.getDepotId()!=null){
            BasicEquipmentEquipmentPool equipmentPool=equipmentPoolService.findByIdObj(storageDeviceget.getDepotId());
            if(equipmentPool != null) storageDeviceget.setDepotName(equipmentPool.getPutPosition());
        }*/
        List<StorageDeviceget> list = new ArrayList<StorageDeviceget>();
        list.add(storageDeviceget);
        return list;
    }

    /**
     * 根据ID获取归还信息
     *
     * @throws Exception
     */
    @RequestMapping(value = "/getDeviceBackById", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据ID获取归还信息", notes = "根据ID获取归还信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "query")
    })
    public List<StorageDeviceback> getDeviceBackById(Integer id) throws Exception {
        List<StorageDeviceback> backList = null;
        if (id != null && !"".equals(id)) {
            backList = deviceBackService.selectBygetId(id);
        }
        return backList;
    }

    /**
     * 获取ID修改信息
     */
    @RequestMapping(value = "/updateDeviceGetById", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "获取ID修改信息", notes = "获取ID修改信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "storageDeviceget", value = "storageDeviceget", paramType = "query")
    })
    public Map<String, Object> updateDeviceGetById(StorageDeviceget storageDeviceget) {
        Map<String, Object> retMap = new HashMap<String, Object>();
        if (storageDeviceget != null) {
            deviceGetService.save(storageDeviceget);
            retMap.put("msg", "success");
        } else {
            retMap.put("msg", "error");
        }
        return retMap;
    }

    /*
     * 增加归还信息
     */
    @RequestMapping(value = "/saveBack", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "增加归还信息", notes = "增加归还信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceBackJson", value = "增加数据", paramType = "query")
    })
    public Map<String, Object> endMore(String deviceBackJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        StorageDeviceget storageDeviceget = null; //领用
        StorageDeviceback storageDeviceback = null; //归还
//        StorageDeviceinput storageDeviceinput = null;

        try {
            //storageDeviceget = (StorageDeviceget)mapper.readValue(deviceBackJson, StorageDeviceget.class);
            storageDeviceback = (StorageDeviceback) mapper.readValue(deviceBackJson, StorageDeviceback.class);
            //增加归还信息（归还表）
            storageDeviceback.setGetId(storageDeviceback.getId());
//           storageDeviceback.setId(0);
            StorageDeviceback storageDeviceback1 = deviceBackService.selectByPrimaryKey(storageDeviceback.getId());
            if (storageDeviceback1 == null) {
                deviceBackService.save(storageDeviceback);
            } else {
                if (storageDeviceback.getBrokenCount()!= null){
                    storageDeviceback1.setBrokenCount(storageDeviceback.getBrokenCount());
                }
                storageDeviceback1.setBackCount(storageDeviceback.getBackCount() + storageDeviceback1.getBackCount());
                deviceBackService.updateByPrimaryKey(storageDeviceback1);
            }
            //领取表中的归还数量增加（领取时，默认归还为0）
            storageDeviceget = deviceGetService.selectByPrimaryKey(storageDeviceback.getGetId());//获取上次归还的数量
            storageDeviceget.setBackCount(storageDeviceget.getBackCount() + storageDeviceback.getBackCount());//归还数量
            deviceGetService.updateByPrimaryKeySelective(storageDeviceget);

            //入库表的数量增加
            //根据设备编号获取入库表的该设备信息
            DSbxxsjyExample example = new DSbxxsjyExample();
            DSbxxsjyExample.Criteria criteria = example.createCriteria();
            criteria.andSbbhEqualTo(storageDeviceback.getNumber());
            DSbxxsjy dSbxxsjy = sblxxjyService.getList(example).get(0);
            dSbxxsjy.setSbsl(dSbxxsjy.getSbsl() + storageDeviceback.getBackCount() - storageDeviceback.getBrokenCount());// 库存数量（可用数量）+归还数量 -故障数量
            dSbxxsjy.setSbms(storageDeviceback.getBrokenCount().toString()); //故障数量
            dSbxxsjy.setMqzt("4");
            sblxxjyService.update(dSbxxsjy);


           /* storageDeviceinput = deviceInputService.selectById(storageDeviceget.getInputId());//获取之前的数量
            //修改现在入库表的数量
            storageDeviceinput.setUseCount(storageDeviceinput.getUseCount() + storageDeviceback.getBackCount() - storageDeviceback.getBrokenCount());// 库存数量（可用数量）+归还数量 -故障数量
            storageDeviceinput.setBrokenCount(storageDeviceback.getBrokenCount());
            if (storageDeviceinput.getBrokenCount() != 0) {
                storageDeviceinput.setStatus(5);//故障
            } else {
                storageDeviceinput.setStatus(0);//正常
            }
            deviceInputService.updateByPrimaryKeySelective(storageDeviceinput);*/
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }


    /**
     * 获取仓房下的器材和设备用于APP端
     */
    @RequestMapping(value = "/getDeviceByHouseId", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "获取仓房下的器材和设备用于APP端", notes = "获取仓房下的器材和设备用于APP端")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "houseId", value = "仓房编号", paramType = "query")
    })
    public PageInfo<StorageDeviceget> getDeviceByHouseId(Integer pageNum, Integer pageSize, Integer houseId) {

        if (pageNum != null && pageSize != null) {
            PageHelper.startPage(pageNum, pageSize);
        }

        List<StorageDeviceget> list = deviceGetService.selectByHouseId(houseId);

        PageInfo<StorageDeviceget> pageInfo = new PageInfo<StorageDeviceget>(list);
        return pageInfo;
    }
}
