package com.chinaitop.depot.device.controller;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.device.model.BasicEquipmentEquipmentPool;
import com.chinaitop.depot.device.model.StorageDeviceinput;
import com.chinaitop.depot.device.model.StorageDeviceinputExample;
import com.chinaitop.depot.device.service.BasicEquipmentEquipmentPoolService;
import com.chinaitop.depot.device.service.StorageDeviceGetService;
import com.chinaitop.depot.device.service.StorageDeviceInputService;
import com.chinaitop.depot.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * 
 * @author gaodd
 *
 */

@RestController
@RequestMapping(value= {"/deviceInput"})
@Api(value= "StorageDeviceInputController", description = "器材设备入库")
public class StorageDeviceInputController {

	@Resource
	private StorageDeviceInputService deviceInputService;
	@Resource
	private BasicEquipmentEquipmentPoolService poolServivce;
	@Resource
	private StorageDeviceGetService deviceGetService;
	
	
	 /**
     * 分页获取器材设备信息
	 * @throws Exception 
     */
	@RequestMapping(value = "/getDeviceList",produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
    @ApiOperation(value="查询入库设备信息", notes = "查询入库设备列表，支持分页")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
  			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
  			@ApiImplicitParam(name = "orgName", value = "当前登录人组织名称", paramType = "query")
  	})
    public PageInfo<StorageDeviceinput> getList(HttpServletRequest request,Integer pageNum, Integer pageSize, String deviceName, Integer deviceType,Integer orgId) throws Exception{
    	StorageDeviceinputExample storageDeviceinputExample=new StorageDeviceinputExample();
    	StorageDeviceinputExample.Criteria criteria=storageDeviceinputExample.createCriteria();
    	if(ParameterUtil.isnotnull(deviceType)){
    		criteria.andDeviceTypeEqualTo(deviceType);
    	}
    	if(ParameterUtil.isnotnull(deviceName)){
    		criteria.andDeviceNameLike(deviceName);
    	}
    	if(ParameterUtil.isnotnull(orgId)){
    		criteria.andOrgIdEqualTo(orgId);
    	}

        //排序
        //storageDeviceinputExample.setOrderByClause("FIELD(status,0,1),input_time desc,id desc");
        storageDeviceinputExample.setOrderByClause("id desc");
        
        if (pageNum!=null && pageSize!=null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<StorageDeviceinput> list = deviceInputService.queryByDevice(storageDeviceinputExample);
        PageInfo<StorageDeviceinput> pageInfo = new PageInfo<StorageDeviceinput>(list);
        return pageInfo;
    }

    
    /**
     * 根据主键id删除数据
     */
	 @RequestMapping(value="/remove", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	 @ApiOperation(value="删除数据", notes = "根据id删除数据")
	 @ApiImplicitParams({
	   	 @ApiImplicitParam(name = "id", value = "id", paramType = "form"),
	 })    
	 public Map<String, Object> remove(Integer id){
        Map<String,Object> retMap = new HashMap<String,Object>();
        deviceInputService.remove(id);
        retMap.put("msg","success");
        return retMap;
    }
	
    /**
     * 新增和修改的数据进行保存
     */
	 @RequestMapping(value="/save",produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	 @ApiOperation(value="保存数据", notes = "保存")
	 @ApiImplicitParams({
	    	@ApiImplicitParam(name="deviceInputJson", value="主表数据", paramType="form")
	 })
    public Map<String, Object> save(String deviceInputJson) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
        	StorageDeviceinput storageDeviceinput = (StorageDeviceinput)mapper.readValue(deviceInputJson, StorageDeviceinput.class);
            storageDeviceinput.setStatus(0);
        	deviceInputService.save(storageDeviceinput);
            modelMap.put("status", "success");
        } catch (IOException e) {
            modelMap.put("status", "error");
            modelMap.put("msg", "保存失败！");
            e.printStackTrace();
        }
        return modelMap;
    }

	   
	 
    /**
     * 根据id加载对应数据
     */
	 @RequestMapping(value ="/loadDataById", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	 @ApiOperation(value="查询数据", notes = "根据id查询数据")
	 @ApiImplicitParams({
		  @ApiImplicitParam(name = "id", value = "id", paramType = "query"),
	 })
    public StorageDeviceinput loadDataById(int id){
	   if(id != 0) {
           return  deviceInputService.loadDataById(id);
       }
       return new StorageDeviceinput();
    }
   
   /**
    * 查询设备数据（5476）
    */
    @RequestMapping(value="/getDevice", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
    @ApiOperation(value="查询设备信息", notes = "查询设备信息")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
  			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
  	})
	public PageInfo<StorageDeviceinput> getDevice(HttpServletRequest request,Integer pageNum, Integer pageSize, String deviceName, Integer deviceTypeId) {

		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		//排序
		HashMap<Object, Object> map = new HashMap<Object, Object>();
		map.put("deviceName", deviceName);
		map.put("deviceTypeId", deviceTypeId);
		List<StorageDeviceinput> list=deviceInputService.queryDevice(map);
		PageInfo<StorageDeviceinput> pageInfo = new PageInfo<StorageDeviceinput>(list);
		return pageInfo;
	}
	 
    /**
     * 根据类型查询数量
     * @param example 粮食业务
     * @return
     */
    @RequestMapping(value="/getDate", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="获取各个类型入库和使用数量", notes = "获取各个类型入库和使用数量")
    public Map<String, Object> getDate() {
		
    	Map<String, Object> returnMap = new HashMap<String, Object>();
    	//入库数量
    	List<Map<String, Object>> queryCountByType = deviceInputService.queryCountByType();
    	String sumCount = "";
    	String useCount = "";
    	if(queryCountByType.size()>0 && queryCountByType!=null){
    		for (Map<String, Object> map : queryCountByType) {
        		/*int useCount = Integer.parseInt(map.get("userCounts").toString());
        		sumCount = sumCount + useCount+",";*/
    			//入库时的数量
    			String a = map.get("sumcounts").toString();
    			int sumCounts = Integer.valueOf(a);
    			
    			//使用数量
    			int useCounts = 0;
    			if(!map.containsKey("brokencount")){
    				 useCounts = sumCounts-(Integer.parseInt(map.get("usercounts").toString()));
    			}else{
    				 useCounts = sumCounts-(Integer.parseInt(map.get("usercounts").toString())+Integer.parseInt(map.get("brokencount").toString()));
    			}
    			
    			sumCount = sumCount + sumCounts+",";
    			useCount = useCount + useCounts+",";
    			
    		}
    		sumCount = sumCount.substring(0, sumCount.length()-1);
    		useCount = useCount.substring(0, useCount.length()-1);
    		System.out.println(sumCount+"****sumCount***");
    		System.out.println(useCount+"****useCount***");
    	}
    	
    	returnMap.put("sumCount", sumCount);
    	
    	/*//使用数量
    	List<Map<String, Object>> queryUseCountByType = deviceGetService.queryCountByType();
    	String useCount = "";
    	if(queryUseCountByType.size()>0 && queryUseCountByType!=null){
    		for (Map<String, Object> map : queryUseCountByType) {
        		int getCount = Integer.parseInt(map.get("getCounts").toString());
        		useCount = useCount + getCount+",";
    		}
        	useCount = useCount.substring(0, useCount.length()-1);
    	}*/
    	
    	returnMap.put("useCount", useCount);
        return returnMap;
    }
    
    
    /**
     * 分页获取器材设备信息
	 * @throws Exception 
     */
	@RequestMapping(value = "/getDeviceAccountList",produces = MediaType.APPLICATION_JSON_VALUE,method=RequestMethod.GET)
    @ApiOperation(value="查询入库设备信息", notes = "查询入库设备列表，支持分页")
  	@ApiImplicitParams({
  			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
  			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
  			@ApiImplicitParam(name = "orgName", value = "当前登录人组织名称", paramType = "query")
  	})
    public PageInfo<StorageDeviceinput> getDeviceAccountList(HttpServletRequest request,Integer pageNum, Integer pageSize, Integer deviceType ,String orgName) throws Exception{
    	StorageDeviceinputExample storageDeviceinputExample=new StorageDeviceinputExample();
    	StorageDeviceinputExample.Criteria criteria=storageDeviceinputExample.createCriteria();
    	if(ParameterUtil.isnotnull(deviceType)){
    		criteria.andDeviceTypeEqualTo(deviceType);
    	}
        //排序
        //storageDeviceinputExample.setOrderByClause("status asc,input_time desc,id desc");
   
        if (pageNum!=null && pageSize!=null) {
            PageHelper.startPage(pageNum, pageSize);
        }
        
        List<StorageDeviceinput> list = deviceInputService.queryDeviceAccount(storageDeviceinputExample);
         for (StorageDeviceinput storageDeviceinput : list) {
        	 if(orgName!=null){
        		 storageDeviceinput.setOrgName(orgName);
        	 }
		}
        PageInfo<StorageDeviceinput> pageInfo = new PageInfo<StorageDeviceinput>(list);
        return pageInfo;
    }
}
